function [XLow, XHi, bestPop] = RangeZoom(fx, XLow, XHi, MaxIters, MaxPop, frac, groups)
% RANGEZOOM narrows the trust region.
%
% INPUT
% =====
% fx - handle of optimized function.
% XLow - array of lower limits of trust region.
% XHi - array of upper limits of trust region.
% MaxIters - maximum number of iterations.
% MaxPop - the population size.
% frac - fraction of test population used to calculate teh mean and
% standard deviation. Default value is 0.1 (10%).
% groups - number of best test population members stored in test population
% matrix per iteraion. Default value is 3.
%
% OUTPUT
% ======
% XLow - array of narrowed lower limits of trust region.
% XHi - array of narrowed upper limits of trust region.
% bestPop - best population obtained by the function. Will be used as the
% initial population by the caller to this function.
% 
if nargin < 7, groups = 3; end
if nargin < 6, frac = 0.1; end
n = length(XLow);
if length(XHi) < n, XHi = XHi(1) + zeros(1,n); end
m = n + 1;
XLow0 = XLow;
XHi0 = XHi;
%---------------------- Narrow Trust Region ----------------
pop = zeros(MaxPop, m);
p = groups;
bestPop = zeros(p*MaxIters,m);
bestPop(:,m) = 1e+99;
for iter =1:p:p*MaxIters
  for i=1:MaxPop
    pop(i,1:n) = XLow + (XHi - XLow) .* rand(1,n);
    pop(i,m) = fx(pop(i,1:n));
  end
  pop = sortrows(pop,m); 
  for i=1:p
    bestPop(iter+i-1,:) = pop(i,:);
  end
end
bestPop = sortrows(bestPop,m);
bestpop = bestPop(1:MaxIters,:);
m2 = fix(MaxPop*frac);
meanX = mean(bestPop(1:m2,1:n));
sdevX = std(bestPop(1:m2,1:n));
XLow = meanX - 2*sdevX;
XHi = meanX + 2*sdevX; 
% check new boundaries with original boundaries
for i=1:n
  if XLow(i) < XLow0(i), XLow(i) = XLow0(i); end
  if XHi(i) > XHi0(i), XHi(i) = XHi0(i); end
end

end

