function [bestX, bestFx] = mc(fx, XLow, XHi, MaxIters)
% Pure random search in trust region range.
% INPUT
% =====
% fx - handle of optimized function.
% XLow - array of lower limits of trust region.
% XHi - array of upper limits of trust region.
% MaxIters - maximum number of iterations.
%
% OUTPUT
% ======
% bestX - array of best solution.
% bestFx - best function value.
%
% Example
% =======
%

  n = length(XLow);
  if length(XHi) < n, XHi = XHi(1) + zeros(1,n); end
  m = n + 1;

  bestX = XLow + (XHi - XLow) .* rand(1,n);
  bestFx = fx(bestX);
  pop = zeros(1,m);
  j = 0;
  for i=1:MaxIters
    X = XLow + (XHi - XLow) .* rand(1,n);
    Fx = fx(X);
    if Fx < bestFx
      bestX = X;
      bestFx = Fx;
      fprintf('%f ', X);
      fprintf('%20.18e at iter %i\n', bestFx, i);
      j = j + 1;
      pop(j,1:n) = X;
      pop(j,m) = Fx;
    end

  end
  
  [rows,~]=size(pop);
  if rows > 10
    fprintf('Means: ');
    disp(mean(pop(rows-10:rows,1:n)));
  else
    disp(mean(pop(:,1:n)));
  end
end
